﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.MetaOptimization {
  [Item("ConstrainedTypeValue", "Represents a type with constraints.")]
  [StorableClass]
  public class ConstrainedTypeValue : TypeValue {

    private IEnumerable<Type> validTypes;
    public IEnumerable<Type> ValidTypes {
      get { return validTypes; }
      set { validTypes = value; }
    }

    public override Type Value {
      set {
        if (!ValidTypes.Contains(value)) throw new NotSupportedException("Value cannot be set. Type is not enlisted in ValidTypes");
        base.Value = value;
      }
    }

    public ConstrainedTypeValue()
      : base() {
      this.readOnly = false;
    }
    public ConstrainedTypeValue(Type value)
      : this() {
      this.Value = value;
    }

    [StorableConstructor]
    protected ConstrainedTypeValue(bool deserializing) : base(deserializing) { }
    protected ConstrainedTypeValue(ConstrainedTypeValue original, Cloner cloner)
      : base(original, cloner) {
      this.validTypes = new List<Type>(original.validTypes);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedTypeValue(this, cloner);
    }
  }

  [Item("ConstrainedTypeValue<>", "Represents a type with constraints.")]
  [StorableClass]
  public class ConstrainedTypeValue<T> : ConstrainedTypeValue where T : class, IItem {

    public ConstrainedTypeValue()
      : base() {
      this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      this.Value = ValidTypes.First();
    }
    public ConstrainedTypeValue(Type value)
      : base() {
      this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      this.Value = value;
    }

    [StorableConstructor]
    protected ConstrainedTypeValue(bool deserializing)
      : base(deserializing) {
      if (deserializing) {
        this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      }
    }

    protected ConstrainedTypeValue(ConstrainedTypeValue original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedTypeValue<T>(this, cloner);
    }
  }
}